#pragma once
#include "Parser.h"
#include "Semantics.h"
#include "ISerializable.h"

namespace ConfigLib
{
	enum OpenMode
	{
		NOT_KEEP = 0,
		KEEP
	};

	//Main config library class (with API for users)
	class CLib
	{
	public:
		CLib(std::string filename, OpenMode mode, IObjectFactory* factory);
		~CLib();

		//Writes serialized object to file
		bool write(ISerializable* obj);
		//Loads object from file
		ISerializable* read();
		//Resets source (to read again or after error)
		void reset();
		//Changes data source (only in NO_KEEP mode)
		void changeDataSource(std::string newSource);
		//Closes data source
		void closeDataSource();

	private:
		std::fstream	file;		//File for writing
		FileDataSource* source;		//Data source
		Parser*			parser;		//Parser
		Semantics*		semant;		//Semantics check and building objects
		OpenMode		oMode;		//Selected open mode
		bool			firstRead;	//Is first read?
	};

}