#pragma once
#include <cassert>
#include <iostream>
#include <string>
#include <fstream>
#include <map>
#include <list>

namespace ConfigLib
{
	//Defines aviable error types
	enum ErrorType
	{
		ET_WARNING = 0,	//Warning - program can continue
		ET_ERROR,			//Error - program can continue, but something may go wrong or be unintended
		ET_FATAL_ERROR		//Fatal error - program can't continue
	};

	//Struct holding position in data source
	struct SourcePos
	{
		int line;				//Line of source
		int posInLine;			//Char in line
		std::streampos filePos;	//Position in file
	};

	struct Error
	{
		ErrorType type;
		SourcePos errorPos;
		std::string message;
	};

	//Singleton class providing easy error output
	class ErrorProvider
	{
	private:
		static ErrorProvider* instance;		//Error provider instance

	public:
		ErrorProvider();
		~ErrorProvider();

		//Returns instance of ErrorProvider if one exists
		static ErrorProvider& instanceOf();

		//Adds error to later output
		void addError(ErrorType type, std::string message, SourcePos pos = SourcePos(), std::string sourceFile = std::string());

		//Outputs all added errors
		void outputErrors();

		//Tells if any error occured
		bool errorsOccured();

	private:
		//Outputs error type in readable form
		void outputErrorType(ErrorType type);

	private:
		std::ifstream sourceFile;						//Error provider file to read lines
		std::map<std::string, std::list<Error>> errs;	//Errors saved to show later
		bool err;										//Flag if error occured
	};

	//Shortcut function to add error
	inline void error(ErrorType type, std::string message, SourcePos pos = SourcePos(), std::string sourceFile = std::string())
	{
		ConfigLib::ErrorProvider::instanceOf().addError(type, message, pos, sourceFile);
	}

	//Shortcut function to output all added errors
	inline void listErrors()
	{
		ConfigLib::ErrorProvider::instanceOf().outputErrors();
	}

	//Shortcut function to check if any error occured
	inline bool wasError()
	{
		return ConfigLib::ErrorProvider::instanceOf().errorsOccured();
	}
}