#pragma once
#include "Scanner.h"

namespace ConfigLib
{
	//Aviable result types
	enum ParserResultType
	{
		OBJ_START = 0,	//Object start
		OBJ_END,		//Object end
		ATTR,			//Attribute
		ARR_START,		//Array start
		ARR_END			//Array end
	};

	//Parser result structure
	struct ParserResult
	{
		ParserResultType type;	//Type of result
		Symbol id;				//Name
		Symbol val;				//Value
	};

	class Parser
	{
	public:
		Parser(FileDataSource* source);
		~Parser();

		//Initialize parser to start parsing (needed before first read from file)
		void init();
		//Parse next object
		void parse();
		//Return reference to parser results
		std::list<ParserResult>* outputPtr();

	private:
		//Loads next symbol from data source using scanner
		void nextSym();
		//Accepts given atom or adds error message
		bool accept(Atom m);
		//Adds error message to ErrorProvider
		void syntaxError(Atom m);
		//Parse <object>
		void object();
		//Parse <attribute>
		void attribute();
		//Parse <value>
		void value();
		//Parse <arr>
		void arr();

	private:
		Symbol sym;						//Loaded symbol
		Scanner* scan;					//Scanner
		FileDataSource* ds;				//Data source to retrieve source file name
		ParserResult wr;				//Working result
		std::list<ParserResult> output;	//Output of parsing
	};
}