#pragma once
#include <string>
#include "ErrorProvider.h"

namespace ConfigLib
{
	//Defines aviable atoms in grammar
	enum Atom
	{
		L_BARCKET = 0,			// [
		R_BRACKET,				// ]
		SEMICOLON,				// ;
		COLON,					// :
		COMMA,					// ,
		L_PARENTHESIS,			// (
		R_PARENTHESIS,			// )
		END_KEYWORD,			// 'End' keyword
		STR_CONST,				// String constant
		INT_CONST,				// Integer constant
		FLOAT_CONST,			// Floating point constant
		ID,						// Identifer
		END_OF_INPUT,			// End of input
		NONE					// None of defined atoms
	};

	//Class holding information about symbol
	class Symbol
	{
	public:
		Symbol();
		Symbol(Atom t, SourcePos p, int v);
		Symbol(Atom t, SourcePos p, double v);
		Symbol(Atom t, SourcePos p, std::string v);
		Symbol(Atom t, SourcePos p);
		~Symbol();

		//Returns symbol type
		Atom getType();
		//Returns int value for INT_CONST
		int getInt();
		//Returns double value for FLOAT_CONST
		double getDouble();
		//Returns string value for STR_CONST
		std::string getStr();
		//Returns position of symbol in source file
		SourcePos getPos();

		//Returns atom in readable form
		static std::string atomToStr(Atom a);

	private:
		Atom type;				//Type of symbol
		int int_value;			//Int const val
		double double_value;	//Float const val
		std::string str_value;	//Str const val
		SourcePos pos;			//Symbol pos in source file
	};
}
